/* Sistemas Integrados TEKNISA

Nome: VALIDADE_ANA

Descrio:
   Retorna a data de validade da anlise conforme cadastrado na tela de Produto
Criao:
29/06/16 - Jurandir Junio */

CREATE OR REPLACE FUNCTION VALIDADE_ANA(P_CDFILIAL   IN FILIAL.CDFILIAL%TYPE,
                                        P_CDFILIRECE IN NOTAFISCAL.CDFILIRECE%TYPE,
                                        P_IDSISTEMA  IN NOTAFISCAL.IDSISTEMA%TYPE,
                                        P_NRLOTEESTQ IN LOTE.NRLOTEESTQ%TYPE,
                                        P_NRSUBLOTE  IN LOTE.NRSUBLOTE%TYPE,
                                        P_IDTIPOANA  IN AUDITORIA.IDTIPOANA%TYPE,
                                        P_CDPRODUTO  IN PRODUTO.CDPRODUTO%TYPE)RETURN DATE
AS
   V_VALIDADE     DATE;          -- Data de validade que ser retornada
   V_DTENTRLOTE   DATE;          -- Data de entrada do lote
   V_DTVALILOTE   DATE;          -- Data de validade do lote
   V_DTINCLUSAO   DATE;          -- Data de incluso da anlise
   V_DTINICIO     DATE;          -- Data de incio da anlise
   V_DTCONCLUSAO  DATE;          -- Data de concluso da anlise 
   V_FORMULA      VARCHAR2(255); -- Frmula da validade da anlise
   V_FORMULAUX    VARCHAR2(255); -- Varivel auxiliar
   V_FORMULAR     VARCHAR2(255); -- Frmula da validade da reanlise
   V_IDVDANALISE  VARCHAR2(1);   -- Verifica se a vd. da anlise  por ms ou deve ser calculada (frmula)
   V_NRDIASVALANA NUMBER;        -- Quantidade dos meses que devem ser considerados na validade da anlise
   V_IDUTILLOTE   VARCHAR2(1);   -- Utiliza controle de lote
   V_NRAUDITORIA  VARCHAR2(10);  -- Nmero da auditoria
   V_FILIAL       VARCHAR2(4);   -- Filial utilizada no clculo

   AUX NUMBER;                   -- Recebe o resultado da funo

BEGIN
   V_VALIDADE := '';

   -- Verifica se a filial controla lote
   BEGIN
      IF (P_IDSISTEMA = 'DFE') THEN
         V_FILIAL := P_CDFILIRECE;

         SELECT IDUTILLOTE
           INTO V_IDUTILLOTE
           FROM PARAMFILIAL
          WHERE CDFILIAL = V_FILIAL;
      ELSE
         V_FILIAL := P_CDFILIAL;

         SELECT IDUTILLOTE
           INTO V_IDUTILLOTE
           FROM PARAMFILIAL
          WHERE CDFILIAL = V_FILIAL;
      END IF;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTILLOTE := 'N';
   END;
   
   -- Traz as informaes para clculo da validade

   SELECT IDVDANALISE, DSFORMULAVDANA, DSFORMULAVDRANA, NRDIASVALANA
     INTO V_IDVDANALISE, V_FORMULA, V_FORMULAR, V_NRDIASVALANA
     FROM PRODUTO
    WHERE CDPRODUTO = P_CDPRODUTO;

   -- Se controla a validade da anlise por meses
   IF (V_IDVDANALISE = '1') THEN
      IF (V_NRDIASVALANA > 0 ) THEN
         V_VALIDADE := SYSDATE + (V_NRDIASVALANA * 30); -- Considerando que um ms sempre ter 30 dias
      ELSE
         V_VALIDADE := '';
      END IF;
   ELSE
      IF (P_IDTIPOANA = 'A') AND (V_FORMULA IS NOT NULL) THEN

         V_FORMULAUX := V_FORMULA;

         -- Verifica se utiliza lote
         IF (V_IDUTILLOTE = 'S') THEN

            BEGIN
               SELECT NVL(DTENTRLOTE, ''), NVL(DTVALILOTE, '')
                 INTO V_DTENTRLOTE, V_DTVALILOTE
                 FROM LOTE
                WHERE CDFILIAL   = V_FILIAL AND
                      NRLOTEESTQ = P_NRLOTEESTQ AND
                      NRSUBLOTE  = NVL(P_NRSUBLOTE, ' ')  AND
                      CDPRODUTO  = P_CDPRODUTO;
            EXCEPTION
               when NO_DATA_FOUND then
                  V_DTENTRLOTE := SYSDATE;
                  V_DTVALILOTE := SYSDATE;
            END;

            -- Vlida se a validade da anlise tem que ser igual a data de validade do lote
            IF (LENGTH(V_FORMULAUX) = 2) AND (INSTR(V_FORMULAUX, 'DV') > 0) THEN
               V_VALIDADE := V_DTVALILOTE;

               -- Validade se a validade da anlise ser no mesmo dia da data de incluso
               IF (INSTR(V_FORMULAUX, 'DI') > 0) THEN
                  V_VALIDADE := SYSDATE;
               END IF;
            ELSE
               -- Substitui as abreviaes pelas datas respectivas
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DV', 'TO_DATE('||''||TO_CHAR(V_DTVALILOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DE', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');

               -- Varivel recebe a expresso. SQL dinmico
               V_FORMULAUX := 'SELECT '||V_FORMULAUX||' FROM DUAL ';

               BEGIN
                  -- Aps o sql for executado e atribudo a varivel, usamos a funo execute immediate para executar a expresso
                  EXECUTE IMMEDIATE V_FORMULAUX INTO AUX;

                  -- Se o valor possuir casas decimais, deve ser acrescentado + 1 e truncar o valor. Por exemplo, 12,2 deve ficar 13.
                  IF (ROUND(AUX)/AUX) = 1 THEN
                     V_VALIDADE := SYSDATE+AUX;
                  ELSE
                     V_VALIDADE := SYSDATE+TRUNC(AUX+1);
                  END IF;
               EXCEPTION
                  /* Se ocorrer algum erro na execuo, provavelmente  porque o select retornou uma data especfica e a varivel AUX
                      do tipo INT. Dessa forma, o sistema tenta retornar a data que foi retornada para a varivel V_VALIDADE. */
                  WHEN OTHERS THEN
                     EXECUTE IMMEDIATE V_FORMULAUX INTO V_VALIDADE;
               END;
            END IF;
         ELSE
            -- Considera a data atual para fazer o clculo da expresso
            V_DTENTRLOTE := SYSDATE;
            V_DTVALILOTE := SYSDATE;

            -- Substitui as abreviaes pelas datas respectivas
            V_FORMULAUX := REPLACE(V_FORMULAUX, 'DV', 'TO_DATE('||''||TO_CHAR(V_DTVALILOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
            V_FORMULAUX := REPLACE(V_FORMULAUX, 'DE', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');

            -- Varivel recebe a expresso. SQL dinmico
            V_FORMULAUX := 'SELECT '||V_FORMULAUX||' FROM DUAL ';

            BEGIN
               -- Aps o sql for executado e atribudo a varivel, usamos a funo execute immediate para executar a expresso
               EXECUTE IMMEDIATE V_FORMULAUX INTO AUX;

               -- Se o valor possuir casas decimais, deve ser acrescentado + 1 e truncar o valor. Por exemplo, 12,2 deve ficar 13.
               IF (ROUND(AUX)/AUX) = 1 THEN
                  V_VALIDADE := SYSDATE+AUX;
               ELSE
                  V_VALIDADE := SYSDATE+TRUNC(AUX+1);
               END IF;
            EXCEPTION
               /* Se ocorrer algum erro na execuo, provavelmente  porque o select retornou uma data especfica e a varivel AUX
                   do tipo INT. Dessa forma, o sistema tenta retornar a data que foi retornada para a varivel V_VALIDADE. */
               WHEN OTHERS THEN
                  EXECUTE IMMEDIATE V_FORMULAUX INTO V_VALIDADE;
            END;
         END IF;
      ELSE    -- Reanlise
         IF (P_IDTIPOANA = 'R') AND (V_FORMULAR IS NOT NULL) THEN
            V_FORMULAUX := V_FORMULAR;

            -- Verifica se utiliza lote
            IF (V_IDUTILLOTE = 'S') THEN

               BEGIN
                  SELECT NVL(DTENTRLOTE, ''), NVL(DTVALILOTE, '')
                    INTO V_DTENTRLOTE, V_DTVALILOTE
                    FROM LOTE
                   WHERE CDFILIAL   = V_FILIAL AND
                         NRLOTEESTQ = P_NRLOTEESTQ AND
                         NRSUBLOTE  = NVL(P_NRSUBLOTE, ' ')  AND
                         CDPRODUTO  = P_CDPRODUTO;
               EXCEPTION
                  WHEN NO_DATA_FOUND then
                     V_DTENTRLOTE := SYSDATE;
                     V_DTVALILOTE := SYSDATE;
               END;

               BEGIN
                  SELECT MAX(NRAUDITORIA) AS NRAUDITORIA, DTINCLAUDIT, DTINIREALAUD, DTFINREALAUD
                    INTO V_NRAUDITORIA, V_DTINCLUSAO, V_DTINICIO, V_DTCONCLUSAO
                    FROM AUDITORIA
                   WHERE CDFILIAL     = V_FILIAL AND
                         CDPRODUTO    = P_CDPRODUTO  AND
                         NRLOTEESTQ   = P_NRLOTEESTQ AND
                         NRSUBLOTE    = NVL(P_NRSUBLOTE, ' ')  AND
                         IDORIGAUDIT  = 'ANA' AND
                         IDSTATUSAUDI = '4'
                    GROUP BY DTINCLAUDIT, DTINIREALAUD, DTFINREALAUD;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_NRAUDITORIA := '';
                     V_DTINCLUSAO  := '';
                     V_DTINICIO    := '';
                     V_DTCONCLUSAO := '';
               END;

               -- Vlida se a validade da anlise tem que ser igual a data de validade do lote
               IF (LENGTH(V_FORMULAUX) = 2) AND (INSTR(V_FORMULAUX, 'DV') > 0) THEN
                  V_VALIDADE := V_DTVALILOTE;
               ELSE

                  -- Substitui as abreviaes pelas datas respectivas
                  V_FORMULAUX := REPLACE(V_FORMULAUX, 'DS', 'TO_DATE('||''||TO_CHAR(V_DTINCLUSAO,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
                  V_FORMULAUX := REPLACE(V_FORMULAUX, 'DV', 'TO_DATE('||''||TO_CHAR(V_DTVALILOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
                  V_FORMULAUX := REPLACE(V_FORMULAUX, 'DE', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
                  V_FORMULAUX := REPLACE(V_FORMULAUX, 'DI', 'TO_DATE('||''||TO_CHAR(V_DTINICIO,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
                  V_FORMULAUX := REPLACE(V_FORMULAUX, 'DC', 'TO_DATE('||''||TO_CHAR(V_DTCONCLUSAO,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');

                  -- Varivel recebe a expresso. SQL dinmico
                  V_FORMULAUX := 'SELECT '||V_FORMULAUX||' FROM DUAL ';

                  BEGIN
                     -- Aps o sql for executado e atribudo a varivel, usamos a funo execute immediate para executar a expresso
                     EXECUTE IMMEDIATE V_FORMULAUX INTO AUX;

                     -- Se o valor possuir casas decimais, deve ser acrescentado + 1 e truncar o valor. Por exemplo, 12,2 deve ficar 13.
                     IF (ROUND(AUX)/AUX) = 1 THEN
                        V_VALIDADE := SYSDATE+AUX;
                     ELSE
                        V_VALIDADE := SYSDATE+TRUNC(AUX+1);
                     END IF;
                  EXCEPTION
                     /* Se ocorrer algum erro na execuo, provavelmente  porque o select retornou uma data especfica e a varivel AUX
                         do tipo INT. Dessa forma, o sistema tenta retornar a data que foi retornada para a varivel V_VALIDADE. */
                     WHEN OTHERS THEN
                        EXECUTE IMMEDIATE V_FORMULAUX INTO V_VALIDADE;
                  END;

               END IF;
            ELSE
               -- Considera a data atual para fazer o clculo da expresso
               V_DTENTRLOTE := SYSDATE;
               V_DTVALILOTE := SYSDATE;

               -- Substitui as abreviaes pelas datas respectivas
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DS', 'TO_DATE('||''||TO_CHAR(V_DTINCLUSAO,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DV', 'TO_DATE('||''||TO_CHAR(V_DTVALILOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DE', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DI', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');
               V_FORMULAUX := REPLACE(V_FORMULAUX, 'DC', 'TO_DATE('||''||TO_CHAR(V_DTENTRLOTE,'''DD/MM/YYYY''')||',''DD/MM/YYYY'')');

               -- Varivel recebe a expresso. SQL dinmico
               V_FORMULAUX := 'SELECT '||V_FORMULAUX||' FROM DUAL ';

               BEGIN
                  -- Aps o sql for executado e atribudo a varivel, usamos a funo execute immediate para executar a expresso
                  EXECUTE IMMEDIATE V_FORMULAUX INTO AUX;

                  -- Se o valor possuir casas decimais, deve ser acrescentado + 1 e truncar o valor. Por exemplo, 12,2 deve ficar 13.
                  IF (ROUND(AUX)/AUX) = 1 THEN
                     V_VALIDADE := SYSDATE+AUX;
                  ELSE
                     V_VALIDADE := SYSDATE+TRUNC(AUX+1);
                  END IF;
               EXCEPTION
                  /* Se ocorrer algum erro na execuo, provavelmente  porque o select retornou uma data especfica e a varivel AUX
                      do tipo INT. Dessa forma, o sistema tenta retornar a data que foi retornada para a varivel V_VALIDADE. */
                  WHEN OTHERS THEN
                     EXECUTE IMMEDIATE V_FORMULAUX INTO V_VALIDADE;
               END;
            END IF;
         ELSE
            -- Se controlar a validade por frmula mas a frmula no ser informada, retorna a data vazia.
            V_VALIDADE := '';
         END IF;
      END IF;
   END IF;

   IF V_VALIDADE < SYSDATE THEN
      RETURN SYSDATE;
   ELSE
      RETURN V_VALIDADE;
   END IF;


-- Se for encontrado algum erro, retorna a data atual
EXCEPTION
   WHEN OTHERS THEN
      RETURN '';
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('VALIDADE_ANA','6.05.xxx'); END;
/

